package de.fhkoeln.fb10.ala.puzzle.domain;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import de.fhkoeln.fb10.ala.puzzle.algorithm.AStarSearch;
import de.fhkoeln.fb10.ala.puzzle.algorithm.Heuristic;
import de.fhkoeln.fb10.ala.puzzle.algorithm.Node;
import de.fhkoeln.fb10.ala.puzzle.algorithm.PuzzleComparator;
import de.fhkoeln.fb10.ala.puzzle.algorithm.PuzzleNodeImpl;

public class AStarPuzzleSolver implements PuzzleSolver {
	
	private Log log = LogFactory.getLog(PuzzleImpl.class);
	
	private Puzzle puzzle;

	private Heuristic heuristic;
	
	public List<Integer> solve() {
		
		List<Integer> steps = new LinkedList<Integer>();
		
		Puzzle tStart = puzzle;
		Puzzle tGoal = new PuzzleImpl(tStart.getDefaultGoalState());
		
		Node tStartNode = new PuzzleNodeImpl(tStart, tGoal, heuristic);
		Node tGoalNode = new PuzzleNodeImpl(tGoal, tGoal, heuristic);

		AStarSearch tAlgo = new AStarSearch();

		long begin = System.currentTimeMillis();
		Node tSolutionNode = tAlgo.solveProblem(tStartNode, tGoalNode, new PuzzleComparator());
		long done = System.currentTimeMillis()-begin;
		
		log.info("SolveProblem executed in " + done + "ms");
		
		PuzzleNodeImpl cur = (PuzzleNodeImpl) tSolutionNode;
		while(cur.getFather()!=null) {
			PuzzleNodeImpl curFather = (PuzzleNodeImpl) cur.getFather();
			Position pos = cur.getState().getPositionOf(Puzzle.KEY_BLANK_STONE);
			steps.add(curFather.getState().getState()[pos.getColumn()][pos.getRow()]);
			cur = (PuzzleNodeImpl) cur.getFather();
		}
		
		Collections.reverse(steps);
		
		log.info("Steps to solve: "  + steps.size());
		log.info("Key-Steps to solve: "  + steps.toString());
		
		return steps;
	}

	public Puzzle getPuzzle() {
		return puzzle;
	}

	public void setPuzzle(Puzzle puzzle) {
		this.puzzle = puzzle;
	}

	public Heuristic getHeuristic() {
		return heuristic;
	}

	public void setHeuristic(Heuristic heuristic) {
		this.heuristic = heuristic;
	}

}
