package de.fhkoeln.fb10.ala.puzzle.domain;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class PuzzleImpl implements Puzzle {
	
	private Log log = LogFactory.getLog(PuzzleImpl.class);
	
	private int[][] mState = null;
	private int mGoalDistance = -1;

	private int size = -1;
	
	private List<PuzzleListener> puzzleObserver;
	
	public PuzzleImpl(int size) {
		this.size = size;
		this.mState = this.getDefaultGoalState();
	}
	
	public PuzzleImpl(int[][] aStateArray) {
		if(!checkArraySize(aStateArray)) {
			throw new IllegalArgumentException("Array not in proper dimension");
		}
		this.mState = aStateArray;
		this.size=aStateArray.length;
	}
	
	private boolean checkArraySize(int[][] aStateArray) {
		int len1 = aStateArray.length;
		boolean tOK = true;
		int i=0;
		while (i<len1 && tOK==true) {
			if (aStateArray[i].length != len1) {
				tOK = false;
			}
			i++;
		}
		return tOK;
	}

	/* (non-Javadoc)
	 * @see IPuzzleState#getPositionOf(int)
	 */
	public Position getPositionOf(int aKey) throws IllegalArgumentException {
		int size = this.getSize();
		for (int i=0; i<size; i++) {
			for (int j=0; j<size; j++) {
				if (mState[i][j] == aKey)
					return new PositionImpl(i,j);
			}
		}
		throw new IllegalArgumentException("Key is not part of the puzzle state.");
	}
	
	public boolean move(int key) {
		Position pos = getPositionOf(key);
		int row = pos.getRow();
		int col = pos.getColumn();
		
		int size = getSize();
		int[][] state = getState(); 
		
		//Integer dir = -1;
		
		if(col-1>=0 && state[col-1][row]==KEY_BLANK_STONE) {
			return moveBlank(Puzzle.DIR_DOWN);
		} else if(col+1<size && state[col+1][row]==KEY_BLANK_STONE) {
			return moveBlank(Puzzle.DIR_UP);
		} else if(row-1>=0 && state[col][row-1]==KEY_BLANK_STONE) {
			return moveBlank(Puzzle.DIR_RIGHT);
		} else if(row+1<size && state[col][row+1]==KEY_BLANK_STONE) {
			return moveBlank(Puzzle.DIR_LEFT);
		} else {
			log.info("Cannot move");
			return false;
		}
	}

	
	/**
	 * Moves the blank field into one of the four given directions
	 * (Example: DIR_UP = moves blank field up = moves field which stays above blank field down into the blankfield)  
	 * @param direction Direction in which to move blank field
	 * @return modified state configuration
	 * TODO make return void
	 */
	public boolean moveBlank(int aDir) {
		Position tPos = this.getPositionOf(Puzzle.KEY_BLANK_STONE);
		
		int tLen = this.getSize();
		int newState[][] = new int[tLen][tLen];
		for (int l=0;l<this.getSize();l++) {
			for (int r=0;r<this.getSize();r++) {
				newState[l][r] = this.mState[l][r];
			}
		}

		int tNeighbour = 0;
		int bColumn = tPos.getColumn();
		
		int brow = tPos.getRow();
		int rDir = -1;
		if (aDir == Puzzle.DIR_LEFT) {
			if (brow > 0) {
				tNeighbour = newState[bColumn][brow-1];
				newState[bColumn][brow-1] = newState[bColumn][brow];
				newState[bColumn][brow] = tNeighbour;
				rDir = Puzzle.DIR_RIGHT;
			} else 
				return false;
		} else
		if (aDir == Puzzle.DIR_RIGHT) {
			if (brow < this.getSize()-1) {
				tNeighbour = newState[bColumn][brow+1];
				newState[bColumn][brow+1] = newState[bColumn][brow];
				newState[bColumn][brow] = tNeighbour;
				rDir = Puzzle.DIR_LEFT;
			} else
				return false;
		} else
		if (aDir == Puzzle.DIR_UP) {
			if (bColumn > 0) {
				tNeighbour = newState[bColumn-1][brow];
				newState[bColumn-1][brow] = newState[bColumn][brow];
				newState[bColumn][brow] = tNeighbour;
				rDir = Puzzle.DIR_DOWN;				
			} else
				return false;
		} else
		if (aDir == Puzzle.DIR_DOWN) {
			if (bColumn < tLen-1) {
				tNeighbour = newState[bColumn+1][brow];
				newState[bColumn+1][brow] = newState[bColumn][brow];
				newState[bColumn][brow] = tNeighbour;
				rDir = Puzzle.DIR_UP;				
			} else
				return false;
		}
		//Direction und key value an das ui
		List<Integer> event = new ArrayList<Integer>(2);
		
		event.add(tNeighbour);
		event.add(rDir);
		notifyObservers(event);
		
		setState(newState);
		
		return true;
	}

	/* (non-Javadoc)
	 * @see puzzle.IPuzzleState#getSons()
	 */
	public PuzzleImpl[] getSons() {
		ArrayList<PuzzleImpl> rList = new ArrayList<PuzzleImpl>();
		
		PuzzleImpl leftSon = new PuzzleImpl(getState());
		if (leftSon.moveBlank(Puzzle.DIR_LEFT))
			rList.add(leftSon);

		PuzzleImpl rightSon = new PuzzleImpl(getState());
		if (rightSon.moveBlank(Puzzle.DIR_RIGHT))
			rList.add(rightSon);

		PuzzleImpl upperSon = new PuzzleImpl(getState());
		if (upperSon.moveBlank(Puzzle.DIR_UP))
			rList.add(upperSon);

		PuzzleImpl lowerSon = new PuzzleImpl(getState());
		if (lowerSon.moveBlank(Puzzle.DIR_DOWN))
			rList.add(lowerSon);

		PuzzleImpl rStates[] = new PuzzleImpl[rList.size()];
		for (int i=0; i<rList.size(); i++) {
			rStates[i] = rList.get(i);
		}

		return rStates;
	}

	public boolean equals(Object obj) {
		if(!(obj instanceof Puzzle)) {
			return super.equals(obj);
		}

		Puzzle tState = (Puzzle)obj;
/*
		try {
			tState = 
		} catch (ClassCastException e) {
			return super.equals(obj);
		}
*/
		if (this.getSize() != tState.getSize())
			throw new IllegalArgumentException();

		int tSA[][] = tState.getState();
		boolean rIsEqual = true;
		for (int l=0;l<this.getSize();l++) {
			for (int r=0;r<this.getSize();r++) {
				if (this.mState[l][r] != tSA[l][r]) {
					rIsEqual = false;					
				}
			}
		}

		return rIsEqual;
	}

	public int[][] getDefaultGoalState() {
		int[][] arr = new int[size][size];
		int k = 1;
		for(int i=0;i<size;i++) {
			for(int j=0;j<size;j++) {
				arr[i][j] = k++;
			}
		}
		arr[size-1][size-1] = KEY_BLANK_STONE;
		return arr;
	}
	
	/* (non-Javadoc)
	 * @see IPuzzleState#getState()
	 */
	public int[][] getState() {
		return this.mState;
	}
	
	/* (non-Javadoc)
	 * @see IPuzzleState#setState(int[][])
	 */
	public void setState(int[][] aStateArray) {
		if (this.checkArraySize(aStateArray)) {
			this.mState = aStateArray;
			this.size = aStateArray.length;
		} else {
			throw new IllegalArgumentException("Exception in Array[n][m]: n does not have same size as m");
		}
	}
	public void shuffle() {
		shuffle(size);
	}
	
	public void shuffle(Integer amount) {
		if(amount!=null) {
			int i = 0;
			int oldDir = DIR_NONE;
			int nextDir;
			while(i<amount) {
				nextDir = 1+(int)Math.floor(Math.random()*4);
				if( (oldDir == DIR_LEFT && nextDir == DIR_RIGHT) ||
					(oldDir == DIR_RIGHT && nextDir == DIR_LEFT) ||
					(oldDir == DIR_UP && nextDir == DIR_DOWN) ||
					(oldDir == DIR_DOWN && nextDir == DIR_UP)) {
					continue;
				}

				if(moveBlank(nextDir)) {
					oldDir = nextDir;
					i++;
				}
			}
		}
	}

	public void notifyObservers(Object event) {
		if(puzzleObserver!=null) {
			for (PuzzleListener cur : puzzleObserver) {
				cur.puzzleUpdate(this,event);
			}
		}
	}
	
	public String toString() {
		String rString = "";
		for (int l=0;l<this.getSize();l++) {
			rString += "|";
			for (int r=0;r<this.getSize();r++) {
				if (this.mState[l][r] != PuzzleImpl.KEY_BLANK_STONE) {
					rString += this.mState[l][r];
				} else {
					rString += " ";
				}
				rString += " |";
			}
			rString += "\n";
		}
		return rString;
	}

	public int compareTo(Puzzle aState) {

		if (this.getSize() != aState.getSize())
			throw new IllegalArgumentException();

		int tSA[][] = aState.getState();
		boolean rIsEqual = true;
		for (int l=0;l<this.getSize();l++) {
			for (int r=0;r<this.getSize();r++) {
				if (this.mState[l][r] != tSA[l][r]) {
					rIsEqual = false;					
				}
			}
		}

		if (rIsEqual)
			return 0;
		else
			return -1;
	}

	public List<PuzzleListener> getPuzzleObserver() {
		return puzzleObserver;
	}

	public void setPuzzleObserver(List<PuzzleListener> puzzleObserver) {
		this.puzzleObserver = puzzleObserver;
	}

	public int getMGoalDistance() {
		return mGoalDistance;
	}
	public void setSize(int size) {
		this.size = size;
	}

	public int getSize() {
		return this.size;
	}

}


