package de.fhkoeln.fb10.ala.puzzle.ui;

import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.Map;

import javax.imageio.ImageIO;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.context.ResourceLoaderAware;
import org.springframework.core.io.Resource;
import org.springframework.core.io.ResourceLoader;

import de.fhkoeln.fb10.ala.puzzle.algorithm.Heuristic;
import de.fhkoeln.fb10.ala.puzzle.domain.PuzzleSolver;
import de.fhkoeln.fb10.ala.puzzle.domain.PuzzleImpl;

public class ControllerPane extends JPanel implements ResourceLoaderAware{
	private static final long serialVersionUID = -3166966216493323275L;

	private Log log = LogFactory.getLog(ControllerPane.class);
	
	private String title;

	private Action solveAction;
	
	private PuzzleImpl puzzle;
	private PuzzlePane puzzlePane;

	private PuzzleSolver puzzleSolver;
	
	private Map<String,Heuristic> heuristics;
	
	//eigentlich String,Resource
	private Map<String,String> imageResources;
	
	private Integer maxValueShuffle;
	private Integer currentValueShuffle;
	private Integer maxValueProblemSize;
	
	private Integer maxValueMovementTime;

	private ResourceLoader resourceLoader;
	
	public void initialize() {
		setLayout(new GridLayout(10,1,2,5));
		
		TitledBorder titledBorder = BorderFactory.createTitledBorder(null, this.title, TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.DEFAULT_POSITION, null, null);
		titledBorder.setTitleFont(new Font("Arial", java.awt.Font.PLAIN, 10));
		setBorder(titledBorder);
		
		//Solve Button
		JButton solveButton = new JButton();
		solveButton.setSize(new java.awt.Dimension(121,16));
		solveButton.setAction(solveAction);
		solveButton.setMnemonic(java.awt.event.KeyEvent.VK_WINDOWS);
		solveButton.setName("calc");
		solveButton.setText("Lsen");
		solveButton.setFont(new java.awt.Font("Arial", java.awt.Font.BOLD, 10));
		
		add(solveButton);

		//Problemgre Label
		final JLabel l1 = new JLabel("Problemgre: ");
		l1.setFont(new java.awt.Font("Arial", java.awt.Font.BOLD, 10));
		
		//Problemgre Slider
		JSlider sizeSlider = new JSlider(2,maxValueProblemSize,4);
		sizeSlider.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent e) {
				JSlider src = (JSlider)e.getSource();
				l1.setText("Problemgre: " + src.getValue());
				puzzle.setSize(src.getValue());
				puzzle.setState(puzzle.getDefaultGoalState());
				puzzlePane.drawPuzzle(false);
			}
			
		});
		add(l1);
		add(sizeSlider);

		//Bewegungen Label
		final JLabel l2 = new JLabel("Anzahl Bewegungen: " + puzzle.getSize());
		l2.setFont(new java.awt.Font("Arial", java.awt.Font.BOLD, 10));

		
		currentValueShuffle = puzzle.getSize();
		//Bewegungen  Mischen 
		final JButton shuffleButton = new JButton();
		shuffleButton.setSize(new java.awt.Dimension(121,16));
		shuffleButton.setMnemonic(java.awt.event.KeyEvent.VK_WINDOWS);
		shuffleButton.setText("Mischen");
		shuffleButton.setFont(new java.awt.Font("Arial", java.awt.Font.BOLD, 10));
		shuffleButton.addMouseListener(new java.awt.event.MouseAdapter() {
			public void mouseClicked(java.awt.event.MouseEvent e) {
				puzzle.shuffle(currentValueShuffle);
			}
		});
		
		add(shuffleButton);
		
		JSlider shuffleSlider = new JSlider(1,maxValueShuffle,puzzle.getSize());
		shuffleSlider.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent e) {
				JSlider src = (JSlider)e.getSource();
				currentValueShuffle = src.getValue();
				l2.setText("Anzahl Bewegungen:" + currentValueShuffle);
			}
			
		});
		add(l2);
		add(shuffleSlider);

		final JLabel l3 = new JLabel("Geschwindigkeit Verschiebung: " + puzzlePane.getMovementTime());
		l3.setFont(new java.awt.Font("Arial", java.awt.Font.BOLD, 10));
		
		JSlider movementSlider = new JSlider(1,maxValueMovementTime,puzzlePane.getMovementTime());
		movementSlider.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent e) {
				JSlider src = (JSlider)e.getSource();
				puzzlePane.setMovementTime(src.getValue());
				l3.setText("Geschwindigkeit Verschiebung: " + (1000-src.getValue()));
			}
		});
		add(l3);
		add(movementSlider);

		String[] hNames = new String[heuristics.size()];
		int i=0;
		for (String curName : heuristics.keySet()) {
			hNames[i++] = curName;
		}

		puzzleSolver.setHeuristic(heuristics.get(hNames[0]));
	
		JComboBox heuristicSelector = new JComboBox(hNames);
		heuristicSelector.addActionListener( new ActionListener() {
			  public void actionPerformed(ActionEvent e) {
			        JComboBox cb = (JComboBox)e.getSource();
			        String selected = (String)cb.getSelectedItem();
			        Heuristic hToUse = heuristics.get(selected);
			        if(hToUse != null) {
			        	puzzleSolver.setHeuristic(hToUse);
			        }
			    } 
		});

		add(heuristicSelector);
		
		String[] iNames = new String[imageResources.size()];
		i=0;
		for (String curName : imageResources.keySet()) {
			iNames[i++] = curName;
		}
		BufferedImage cur;
		try {
			Resource res = resourceLoader.getResource(imageResources.get(iNames[0]));
			cur = ImageIO.read(res.getURL());
			puzzlePane.setImage(cur);
		} catch (IOException e2) {
			log.error(e2);
		}
		
		JComboBox imageSelector = new JComboBox(iNames);
		imageSelector.addActionListener( new ActionListener() {
			  public void actionPerformed(ActionEvent e) {
			        JComboBox cb = (JComboBox)e.getSource();
			        String selected = (String)cb.getSelectedItem();
			        BufferedImage cur = null;
			        if(imageResources.get(selected)!=null) {
		    			try {
		    				cur = ImageIO.read(resourceLoader.getResource(imageResources.get(selected)).getURL());
						} catch (IOException e1) {
							log.error(e1);
						}
			        }
					puzzlePane.setImage(cur);
			        puzzlePane.drawPuzzle();
			    } 
		});

		add(imageSelector);		
		
		setVisible(true);
		
	}
	
	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public Action getSolveAction() {
		return solveAction;
	}

	public void setSolveAction(Action solveAction) {
		this.solveAction = solveAction;
	}

	public PuzzleImpl getPuzzle() {
		return puzzle;
	}

	public void setPuzzle(PuzzleImpl puzzle) {
		this.puzzle = puzzle;
	}

	public PuzzlePane getPuzzlePane() {
		return puzzlePane;
	}

	public void setPuzzlePane(PuzzlePane puzzlePane) {
		this.puzzlePane = puzzlePane;
	}

	public Integer getMaxValueProblemSize() {
		return maxValueProblemSize;
	}

	public void setMaxValueProblemSize(Integer maxValueProblemSize) {
		this.maxValueProblemSize = maxValueProblemSize;
	}

	public Integer getMaxValueMovementTime() {
		return maxValueMovementTime;
	}

	public void setMaxValueMovementTime(Integer maxValueMovementTime) {
		this.maxValueMovementTime = maxValueMovementTime;
	}

	public Integer getMaxValueShuffle() {
		return maxValueShuffle;
	}

	public void setMaxValueShuffle(Integer maxValueShuffle) {
		this.maxValueShuffle = maxValueShuffle;
	}

	public PuzzleSolver getPuzzleSolver() {
		return puzzleSolver;
	}

	public void setPuzzleSolver(PuzzleSolver puzzleSolver) {
		this.puzzleSolver = puzzleSolver;
	}

	public Map getHeuristics() {
		return heuristics;
	}

	public void setHeuristics(Map<String, Heuristic> heuristics) {
		this.heuristics = heuristics;
	}

	public Map<String, String> getImageResources() {
		return imageResources;
	}

	public void setImageResources(Map<String, String> puzzleImages) {
		this.imageResources = puzzleImages;
	}

	public void setResourceLoader(ResourceLoader arg0) {
		this.resourceLoader = arg0;
	}

}
