package de.fhkoeln.fb10.ala.puzzle.test;

import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;


import junit.framework.TestCase;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.LogarithmicAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.text.TextUtilities;

import com.lowagie.text.Document;
import com.lowagie.text.DocumentException;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.DefaultFontMapper;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfTemplate;
import com.lowagie.text.pdf.PdfWriter;

import de.fhkoeln.fb10.ala.puzzle.algorithm.AStarSearch;
import de.fhkoeln.fb10.ala.puzzle.algorithm.ManhattenHeuristicImpl;
import de.fhkoeln.fb10.ala.puzzle.algorithm.Heuristic;
import de.fhkoeln.fb10.ala.puzzle.algorithm.SimpleHeuristicImpl;
import de.fhkoeln.fb10.ala.puzzle.algorithm.Node;
import de.fhkoeln.fb10.ala.puzzle.algorithm.PuzzleComparator;
import de.fhkoeln.fb10.ala.puzzle.algorithm.PuzzleNodeImpl;
import de.fhkoeln.fb10.ala.puzzle.domain.Puzzle;
import de.fhkoeln.fb10.ala.puzzle.domain.PuzzleImpl;

public class AStarPowerTest extends TestCase {

	public void testLaufzeitVergleichTest() {
		Integer UMFANG = 10; //Anzahl der Versuche ber denen ein Mittelwert gebildet wird
		Integer MAXPROBLEMSIZE = 15;
		boolean MANHATTAN_HEURISTIC = true;
		Integer GAMESIZE = 4;

		MANHATTAN_HEURISTIC = false;
		laufzeitVergleichTest(UMFANG, MAXPROBLEMSIZE, MANHATTAN_HEURISTIC, GAMESIZE);

		MANHATTAN_HEURISTIC = true;
		laufzeitVergleichTest(UMFANG, MAXPROBLEMSIZE, MANHATTAN_HEURISTIC, GAMESIZE);

		assertTrue(true);
	}

	private void laufzeitVergleichTest(Integer UMFANG, Integer MAXPROBLEMSIZE, boolean MANHATTAN_HEURISTIC, Integer GAMESIZE) {

		/** the following line is a workaround for JDK 1.5 (fix by Adriaan Joubert) */
		TextUtilities.setUseDrawRotatedStringWorkaround(false);

		Double[][] resultValues = new Double[MAXPROBLEMSIZE][2]; //aktueller Versuch

		for(int j=0;j<MAXPROBLEMSIZE;j++) {
			//Arrays.fill(resultValues[j], new Double(0));
			resultValues[j][0] = new Double(j+1); //x mit jeweiliger problemgroesse initiailisieren
			resultValues[j][1] = new Double(0); //y mit StartWert 0 initilaisieren
		}

		String tHeuristicName = "";
		if (MANHATTAN_HEURISTIC) {
			tHeuristicName = "Manhattan";
		} else {
			tHeuristicName = "Simple";
		}

		System.out.println("Starte Datensammlung fr " + tHeuristicName + "-Heuristic. Maximale Problemgroesse: " + MAXPROBLEMSIZE);

		for(int j=0; j<UMFANG; j++) {
			System.out.println("Datensammlung: Stichprobe " + (j+1) + " von " + UMFANG + " ...");
			for(int i=1; i<=MAXPROBLEMSIZE; i++) {
				Long time = stopSolvingTime(GAMESIZE, i, MANHATTAN_HEURISTIC);
				resultValues[i-1][1] += new Double(time); //y
			}
		}

		//Mittelwerte
		for(int i=0;i<MAXPROBLEMSIZE;i++) {
			resultValues[i][1] /= UMFANG;
			//System.out.println("n = " + (i+1) + "   time = " + resultValues[i][1] +  "     x = " + Math.log(i+1) + "  y = " + Math.log(resultValues[i][1]));
		}

		JFreeChart jChart = buildChart(resultValues, "Powertest - " + GAMESIZE + "x" + GAMESIZE + ", "+ UMFANG + " Proben, " + MAXPROBLEMSIZE + " Pfadlnge, " + tHeuristicName + "-Heuristic ", "Pfadlnge der Lsung", "Anzahl traversierter Knoten", Boolean.TRUE);


		String tFileName = "Matrix " + GAMESIZE + "x" + GAMESIZE +" Loesungslaenge " + MAXPROBLEMSIZE + " Stichproben " + UMFANG + " " + tHeuristicName + ".pdf";
		convertToPdf(jChart, 595, 300, tFileName);

	}

	private Long stopSolvingTime(Integer puzzleSize, Integer permutationen, boolean manhattanheuristik) {

		PuzzleNodeImpl tStartNode = null;
		PuzzleNodeImpl tGoalNode = null;

		Heuristic tHeuristic = null;
		if (manhattanheuristik) {
			tHeuristic = new ManhattenHeuristicImpl();
		} else {
			tHeuristic = new SimpleHeuristicImpl();
		}

		AStarSearch tAlgo = new AStarSearch();
		//
		Node tSolutionNode = null;
		PuzzleImpl tPuzzleStart = null;
		Puzzle tPuzzleGoal = null;
		Heuristic tManhattan = new ManhattenHeuristicImpl();

		do {
			tPuzzleStart = new PuzzleImpl(puzzleSize);
			tPuzzleStart.shuffle(permutationen);
			tPuzzleGoal = new PuzzleImpl(tPuzzleStart.getDefaultGoalState());
			PuzzleNodeImpl tStartNodeManh = new PuzzleNodeImpl(tPuzzleStart, tPuzzleGoal, tManhattan);
			PuzzleNodeImpl tGoalNodeManh = new PuzzleNodeImpl(tPuzzleGoal, tPuzzleGoal, tManhattan);

			tSolutionNode = tAlgo.solveProblem(tStartNodeManh, tGoalNodeManh, new PuzzleComparator());
			//System.out.println("Shuffle: " + permutationen + " Pfadlnge: " + tSolutionNode.getPathcost());
		} while (tSolutionNode.getPathcost() != permutationen);

		tStartNode = new PuzzleNodeImpl(tPuzzleStart, tPuzzleGoal, tHeuristic);
		tGoalNode = new PuzzleNodeImpl(tPuzzleGoal, tPuzzleGoal, tHeuristic);

		//long timeBegin = System.currentTimeMillis();
		if (! manhattanheuristik) {
			tPuzzleStart = new PuzzleImpl(puzzleSize);
			tPuzzleStart.shuffle(permutationen);
			tPuzzleGoal = new PuzzleImpl(tPuzzleStart.getDefaultGoalState());
			tAlgo = new AStarSearch();
			tSolutionNode = tAlgo.solveProblem(tStartNode, tGoalNode, new PuzzleComparator());
		}


		//System.out.println("Shuffle: " + permutationen + " Pfadlnge: " + tSolutionNode.getPathcost());
		//long timeDone = System.currentTimeMillis();

		assertEquals(tGoalNode,tSolutionNode); //test ob richtig gelst

		return tAlgo.countNodes();
	}
    /**
     * Gets an example XY chart
     * @return an XY chart
     */
    private JFreeChart buildChart(Double[][] values,
    		String title, String xAxis, String yAxis, Boolean logAxis ) {
    	XYSeries series = new XYSeries(title);

    	for(int i=0;i<values.length;i++) {
        	series.add(values[i][0], values[i][1]);
    	}
    	XYSeriesCollection dataset = new XYSeriesCollection();
    	dataset.addSeries(series);


    	JFreeChart chart = ChartFactory.createXYLineChart(
    			title, xAxis, yAxis, dataset,
				PlotOrientation.VERTICAL, false, false, false);

		if(logAxis) {
	    	XYPlot plot = chart.getXYPlot();
		    NumberAxis domainAxis = new LogarithmicAxis(xAxis);
		    NumberAxis rangeAxis = new LogarithmicAxis(yAxis);
		    plot.setDomainAxis(domainAxis);
		    plot.setRangeAxis(rangeAxis);
		}
	    return chart;
    }

    /**
     * Converts a JFreeChart to PDF syntax.
     * @param filename	the name of the PDF file
     * @param chart		the JFreeChart
     * @param width		the width of the resulting PDF
     * @param height	the height of the resulting PDF
     */
    private void convertToPdf(JFreeChart chart, int width, int height, String filename) {
    	// step 1
    	Document document = new Document(new Rectangle(width, height));
    	try {
    		// step 2
    		PdfWriter writer;
			writer = PdfWriter.getInstance(document, new FileOutputStream(filename));
			// step 3
    		document.open();
    		// step 4
    		PdfContentByte cb = writer.getDirectContent();
    		PdfTemplate tp = cb.createTemplate(width, height);
    		Graphics2D g2d = tp.createGraphics(width, height, new DefaultFontMapper());
    		Rectangle2D r2d = new Rectangle2D.Double(0, 0, width, height);
    		chart.draw(g2d, r2d);
    		g2d.dispose();
    		cb.addTemplate(tp, 0, 0);
    	}
    	catch(DocumentException de) {
    		de.printStackTrace();
    	}
    	catch (FileNotFoundException e) {
    		e.printStackTrace();
    	}
    	// step 5
    	document.close();
    }

}