package de.fhkoeln.fb10.ala.puzzle.test;

import java.util.Arrays;
import java.util.List;

import junit.framework.TestCase;
import de.fhkoeln.fb10.ala.puzzle.domain.Puzzle;
import de.fhkoeln.fb10.ala.puzzle.domain.PuzzleImpl;

public class PuzzleStateTest extends TestCase {

	/*
	 * Test method for 'de.fhkoeln.fb10.ala.puzzle.domain.PuzzleState.getDefaultGoalState()'
	 */
	public void testGetDefaultGoalState() {
		int size=3;
		int[][] arr = new int[size][size];
		int k = 1;
		for(int i=0;i<size;i++) {
			for(int j=0;j<size;j++) {
				arr[i][j] = k++;
			}
		}
		arr[size-1][size-1] = PuzzleImpl.KEY_BLANK_STONE;
		
		PuzzleImpl p = new PuzzleImpl(3);
		p.shuffle(3*3);

		int[][] defaultPuzzle = p.getDefaultGoalState();

		assertTrue(Arrays.deepEquals(defaultPuzzle, arr));
	}
	
	public void testMoveBlank() {
		int startArray[][] = {{4, 7, 5},
				{1, Puzzle.KEY_BLANK_STONE, 8},
				{6, 2, 3}};

		int move2Up[][] = {{4, 7, 5},
				{1, 2, 8},
				{6, Puzzle.KEY_BLANK_STONE, 3}};

		int move7Down[][] = {{4, Puzzle.KEY_BLANK_STONE, 5},
				{1, 7, 8},
				{6, 2, 3}};

		int move1Left[][] = {{4, 7, 5},
				{Puzzle.KEY_BLANK_STONE, 1, 8},
				{6, 2, 3}};

		int move8Right[][] = {{4, 7, 5},
				{1, 8, Puzzle.KEY_BLANK_STONE},
				{6, 2, 3}};

		PuzzleImpl[] expectedSons = new PuzzleImpl[4]; 
		expectedSons[0] = new PuzzleImpl(move1Left);
		expectedSons[1] = new PuzzleImpl(move8Right);
		expectedSons[2] = new PuzzleImpl(move2Up);
		expectedSons[3] = new PuzzleImpl(move7Down);
		
		PuzzleImpl p = new PuzzleImpl(startArray);

		int[][] oldState = p.getState();
		
		p.moveBlank(PuzzleImpl.DIR_LEFT);
		assertTrue(Arrays.deepEquals(p.getState(),move1Left));
		p.moveBlank(PuzzleImpl.DIR_RIGHT);
		assertTrue(Arrays.deepEquals(p.getState(),oldState));

		p.moveBlank(PuzzleImpl.DIR_RIGHT);
		assertTrue(Arrays.deepEquals(p.getState(),move8Right));
		p.moveBlank(PuzzleImpl.DIR_LEFT);
		assertTrue(Arrays.deepEquals(p.getState(),oldState));

		p.moveBlank(PuzzleImpl.DIR_DOWN);
		assertTrue(Arrays.deepEquals(p.getState(),move2Up));
		p.moveBlank(PuzzleImpl.DIR_UP);
		assertTrue(Arrays.deepEquals(p.getState(),oldState));

		p.moveBlank(PuzzleImpl.DIR_UP);
		assertTrue(Arrays.deepEquals(p.getState(),move7Down));
		p.moveBlank(PuzzleImpl.DIR_DOWN);
		assertTrue(Arrays.deepEquals(p.getState(),oldState));
	}
	
	public void testMove() {
		
		int startArray[][] = {{4, 7, 5},
		  {1, Puzzle.KEY_BLANK_STONE, 8},
		  {6, 2, 3}};

		int move2Up[][] = {{4, 7, 5},
				  {1, 2, 8},
				  {6, Puzzle.KEY_BLANK_STONE, 3}};

		int move7Down[][] = {{4, Puzzle.KEY_BLANK_STONE, 5},
				{1, 7, 8},
				{6, 2, 3}};

		int move1Left[][] = {{4, 7, 5},
				{Puzzle.KEY_BLANK_STONE, 1, 8},
				{6, 2, 3}};

		int move8Right[][] = {{4, 7, 5},
				{1, 8, Puzzle.KEY_BLANK_STONE},
				{6, 2, 3}};

		
		PuzzleImpl p = new PuzzleImpl(startArray);
		int[][] oldState = p.getState();
		
		p.move(5);
		p.move(4);
		p.move(6);		
		p.move(3);		
		
		assertTrue(Arrays.deepEquals(oldState,p.getState()));
		
		p.move(2);
		assertTrue(Arrays.deepEquals(move2Up,p.getState()));
		
		p.move(2);
		assertTrue(Arrays.deepEquals(oldState,p.getState()));
		
		p.move(7);
		assertTrue(Arrays.deepEquals(move7Down,p.getState()));
		
		p.move(7);
		assertTrue(Arrays.deepEquals(oldState,p.getState()));
	
		p.move(1);
		assertTrue(Arrays.deepEquals(move1Left,p.getState()));
		
		p.move(1);
		assertTrue(Arrays.deepEquals(oldState,p.getState()));

		p.move(8);
		assertTrue(Arrays.deepEquals(move8Right,p.getState()));
		
		p.move(8);
		assertTrue(Arrays.deepEquals(oldState,p.getState()));
		
	}

	public void testSons() {
		int startArray[][] = {{4, 7, 5},
				{1, Puzzle.KEY_BLANK_STONE, 8},
				{6, 2, 3}};

		int move2Up[][] = {{4, 7, 5},
				{1, 2, 8},
				{6, Puzzle.KEY_BLANK_STONE, 3}};

		int move7Down[][] = {{4, Puzzle.KEY_BLANK_STONE, 5},
				{1, 7, 8},
				{6, 2, 3}};

		int move1Left[][] = {{4, 7, 5},
				{Puzzle.KEY_BLANK_STONE, 1, 8},
				{6, 2, 3}};

		int move8Right[][] = {{4, 7, 5},
				{1, 8, Puzzle.KEY_BLANK_STONE},
				{6, 2, 3}};

		PuzzleImpl[] expectedSons = new PuzzleImpl[4]; 
		expectedSons[0] = new PuzzleImpl(move1Left);
		expectedSons[1] = new PuzzleImpl(move8Right);
		expectedSons[2] = new PuzzleImpl(move7Down);
		expectedSons[3] = new PuzzleImpl(move2Up);
		
		PuzzleImpl p = new PuzzleImpl(startArray);
		int[][] oldState = p.getState();
		
		PuzzleImpl[] pSons = (PuzzleImpl[]) p.getSons();
		
		// check if state is not touched
		assertEquals(oldState,p.getState());
		
		assertTrue(Arrays.deepEquals(pSons,expectedSons));
		
	}
}
